# 中西部区域赛os内核实现赛道初赛记录

[![Lang](https://img.shields.io/badge/Lang-Rust-green)](https://img.shields.io/badge/Lang-Rust-green)  [![ISA](https://img.shields.io/badge/ISA-riscv64-yellowgreen)](https://img.shields.io/badge/ISA-riscv64-yellowgreen)  [![Platform](https://img.shields.io/badge/Platform-Qemu%2c%20k210%20%26%20Hifive%20Unmatched-yellow)](https://img.shields.io/badge/Platform-Qemu%2c%20k210%20%26%20Hifive%20Unmatched-yellow) 

### 参赛队伍介绍
队名：lwx
队员：李惟鑫

### 演示视频
链接：https://pan.baidu.com/s/1Um67KHdPhmsGMWWHMYNdBw
提取码：lwos

### 实验详情

+ 初赛要求实现功能

根据官方给出的测试文档，我们可以看到，我们的内核要完成很多的新增系统调用。测试文档中，一共需要实现以下的系统调用：
（1） test_openat
（2） test_dup2
（3） test_pipe
（4） test_brk 
（5） test_chdir
（6） test_clone
（7） test_dup
（8） test_execve
（9） test_exit
（10） test_fork
（11） test_fstat
（12） test_getcwd
（13） test_getdents
（14） test_getpid
（15） test_gettimeofday
（16） test_mkdir
（17） test_mmap
（18） test_mount
（19） test_munmap
（20） test_open
（21） test_read 
（22） test_times
（23） test_umount
（24） test_uname
（25） test_unlink
（26） test_wait
（27） test_waitpid
（28） test_write
（29） test_yield
（30） test_sleep 

+ 个人提交

我使用的是https://gitlab.eduxiji.net/2019301887/oskernel2022-npucore  提供的22年的NPU_CORE的代码为基础进行修改加入新的测试样例来进行修改。一些系统调用在22版已经实现，如：clone，mmap,exit,unlink等，在这里就可以在前人的基础上进行添加

+ 实现过程

（1）首先需要在原有的基础上，添加新的系统调用shut_down。需要模仿头歌实验二第一个实验一样，在os/src/syscall/mod.rs和process.rs,user/src/syscall.rs和user_call.rs中添加对应的函数，并且相互调用。
（2）区域赛实验测试样例的添加
刷入 bash initproc
核心在于`preload_app.S`文件和`flush_preload`函数。
`preload_app.S`文件在`data`段定义了`initproc`和`bash`的起始和终止位置，并指定了二者二进制文件的相对路径，这番操作将`initproc`和`bash`的内容预加载到了内核的`data`段中。
```asm
# os\src\preload_app.S
    .section .data
    .global sinitproc
    .global einitproc
    .align 12
sinitproc:
    .incbin "../user/target/riscv64gc-unknown-none-elf/release/initproc"
einitproc:
    .align 12

    .section .data
    .global sbash
    .global ebash
    .align 12
sbash:
    .incbin "../bash-5.1.16/bash"
ebash:
    .align 12
flush_preload函数引用了preload_app.S中定义的符号，创建initproc和bash文件，将地址中的内容分别写入对应的文件里，实现了bash和initproc的加载。
// os\src\fs\mod.rs
pub fn flush_preload() {
    extern "C" {
        fn sinitproc();
        fn einitproc();
        fn sbash();
        fn ebash();
    }
    // 省略打印地址信息的代码
    let initproc = ROOT_FD.open("initproc", OpenFlags::O_CREAT, false).unwrap();
    initproc.write(None, unsafe {
        core::slice::from_raw_parts(
            sinitproc as *const u8,
            einitproc as usize - sinitproc as usize,
        )
    });
    for ppn in crate::mm::PPNRange::new(
        crate::mm::PhysAddr::from(sinitproc as usize).floor(),
        crate::mm::PhysAddr::from(einitproc as usize).floor(),
    ) {
        crate::mm::frame_dealloc(ppn);
    }
    let bash = ROOT_FD.open("bash", OpenFlags::O_CREAT, false).unwrap();
    bash.write(None, unsafe {
        core::slice::from_raw_parts(sbash as *const u8, ebash as usize - sbash as usize)
    });
    for ppn in crate::mm::PPNRange::new(
        crate::mm::PhysAddr::from(sbash as usize).floor(),
        crate::mm::PhysAddr::from(ebash as usize).floor(),
    ) {
        crate::mm::frame_dealloc(ppn);
    }
}
实现了两个上述功能需要在内核加入：
core::arch::global_asm!(include_str!("preload_app.S"));
并在主函数中调用：
fs::flush_preload();
内存管理 在 Npucore 中，虚拟地址空间和物理地址空间均采用页式管理，且每个页面的大小 为 4KiB。如此一来，一个虚拟页面中的数据正好对应存储在一个物理页帧上，便 于管理。根据页面大小的规定可知，每个页面需要使用 12 位字节地址来进行页内索引。 根据页式管理的知识，我们将虚拟地址和物理地址均分成两部分：它们的低 12 位，即 [11:0] 被称为页内偏移 (Page Offset)，它描述一个地址指向的字节在其所在页面中的相 对位置。在 SV39 分页模式下，我们规定虚拟地址一共 39 位，则虚拟地址的高 27 位，即 [38:12] 为它的虚拟页号 VPN (Virtual Page Number)；我们规定物理地址一共 56 位，则物 理地址的高 44 位，即 [55:12] 为它的物理页号 PPN (Physical Page Number)。我们也将物理地址以 64 位进行封装。具体的实现如下：
// Definitions
#[repr(C)]
#[derive(Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub struct PhysAddr(pub usize);

#[repr(C)]
#[derive(Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub struct VirtAddr(pub usize);

#[repr(C)]
#[derive(Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub struct PhysPageNum(pub usize);

#[repr(C)]
#[derive(Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
pub struct VirtPageNum(pub usize);

上面分别给出了物理地址 PA、虚拟地址 VA、物理页号 PPN、虚拟页号 VPN 的类
型声明，它们都是元组式结构体，可以看成 usize 的一种简单包装。

地址的取整：
impl PhysAddr {
pub fn floor(&self) -> PhysPageNum {
PhysPageNum(self.0 / PAGE_SIZE)
}
pub fn ceil(&self) -> PhysPageNum {
PhysPageNum((self.0 + PAGE_SIZE - 1) / PAGE_SIZE)
}
}
floor 为下取整方法，而 ceil 为上取整方法。其中，PAGE_SIZE 为 4096，表示每个
页面的大小。
接下来介绍地址与页号的转换，我们同样是实现 From<T> trait：
impl PhysAddr {
pub fn page_offset(&self) -> usize { self.0 & (PAGE_SIZE - 1) }
}
impl From<PhysAddr> for PhysPageNum {
fn from(v: PhysAddr) -> Self {
assert_eq!(v.page_offset(), 0);
v.floor()
}
}
impl From<PhysPageNum> for PhysAddr {
fn from(v: PhysPageNum) -> Self { Self(v.0 << PAGE_SIZE_BITS) }
}
pub struct PageTable {
root_ppn: PhysPageNum ,
frames: Vec<FrameTracker >,
}

impl PageTable {
pub fn new() -> Self {
let frame = frame_alloc().unwrap();
PageTable {
root_ppn: frame.ppn,
frames: vec![frame],
}
}
}

可见，PageTable 类型保存了其所在的物理页帧的页号作为其唯一标识。此外还有
一个 frames 字段，该字段主要用于实现页表映射的物理页帧的生命周期与页表同步
由于页表使用物理页号进行标识，因此我们已经可以方便地使用 PTE 来寻找对应
的页表位置，即非叶子节点的页表中的 PTE 存储的 PPN，即为一个页表的 root_ppn。那
么，我们怎么通过页表来取得其存储的 PTE 呢？需要用到以下方法：
impl PhysPageNum {
pub fn get_pte_array(&self) -> &’static mut [PageTableEntry] {
let pa: PhysAddr = self.clone().into();
unsafe {
core::slice::from_raw_parts_mut(pa.0 as *mut PageTableEntry ,
512)
}
}
该方法构造可变引用来直接访问一个物理页号对应的物理页帧，然后正如图 4-3 的
第一个箭头那样，将页表所在的物理页帧切分成 512 份，每一份正是对应一项 PTE，最
终生成一个页表项类型的定长数组（长度为 512）的可变引用，代表了多级页表
中的一个节点。至此，我们也实现了从一个页表获取其页表项的方法。
我们使用 PageTable 类型的 translate_va 方法来实现虚拟地址到物理
地址的转换：
pub fn translate_va(&self, va: VirtAddr) -> Option<PhysAddr> {
self.find_pte(va.clone().floor()).map(|pte| {
let aligned_pa: PhysAddr = pte.ppn().into();
let offset = va.page_offset();
let aligned_pa_usize: usize = aligned_pa.into();
(aligned_pa_usize + offset).into()
})
}
该方法的核心实际上是调用了 find_pte 方法，完成从虚拟页号查询到叶子节点的对
应 PTE 的过程。第 3 行是从最终查询到的 PTE 获取物理地址的 PPN 段，而 4、5、6 行
实际上是完成了一个物理地址的拼接过程。map 是一个泛型闭包，将最后拼接而成的
usize 类型转化为 PhysAddr 类型。我们接下来看 find_pte 方法的实现：
fn find_pte(&self, vpn: VirtPageNum) -> Option<&PageTableEntry > {
let idxs = vpn.indexes();
let mut ppn = self.root_ppn;
let mut result: Option<&PageTableEntry > = None;
for i in 0..3 {
let pte = &ppn.get_pte_array()[idxs[i]];
if !pte.is_valid() {
return None;
}
if i == 2 {
result = Some(pte);
break;
}
ppn = pte.ppn();
}
result
}
由于 find_pte 是一个页表类型下的方法，其在调用时对应一个页表实例。而由于每
个地址空间总是保存着其根页表的位置信息（satp 寄存器），因此该方法的调用者总是
根页表，相当于我们已经实现了取得根页表的第一步。因此本方法实际就是实现对三级
页表树的查询。
第 2 行，调用 indexes 方法:
impl VirtPageNum {
pub fn indexes(&self) -> [usize; 3] {
let mut vpn = self.0;
let mut idx = [0usize; 3];
for i in (0..3).rev() {
idx[i] = vpn & 511;
vpn >>= 9;
}
idx
}
sbrk 系统调用是早期的 Unix 系统中的一个系统调用，用于动态的分配用户空间。
sbrk 系统调用的原型如下：
void *sbrk(intptr_t increment);
sbrk 系统调用将堆的大小增加 increment 字节，并返回堆的起始地址。如果 increment 为
负数，则堆的大小减少 increment 字节。如果堆的大小超过了进程的地址空间，则 sbrk
系统调用返回-1，并设置 errno 为 ENOMEM。sbrk 系统调用可以为一个进程扩大或者缩
小堆的大小，主要的实现是由 os/src/memory_set.rs 中的 sbrk 函数完成。sbrk 函数调用
Memoryset::mmap 或者 Memoryset::munmap 来实现堆的扩大或者缩小。mmap 函数不仅
用于 sbrk 系统调用，还用于 mmap 系统调用，用于将文件映射到用户空间和开辟匿名内
存映射。
```rust
pub fn sbrk(&mut self, heap_pt: usize, heap_bottom: usize, increment:
isize) -> usize {
let old_pt: usize = heap_pt;
let new_pt: usize = old_pt + increment as usize;
// 判 断 扩 大 堆 还 是 缩 小 堆
if increment > 0 {
let limit = heap_bottom + USER_HEAP_SIZE;
if new_pt > limit {
return old_pt;
} else {
self.mmap(
old_pt,
increment as usize,
MapPermission::R | MapPermission::W | MapPermission::U,
MapFlags::MAP_ANONYMOUS | MapFlags::MAP_FIXED |
MapFlags::MAP_PRIVATE ,
1usize.wrapping_neg(),
0,
);
trace!(”[sbrk] heap area expanded to {:X}”, new_pt);
}
} else if increment < 0 {
// 如 果 缩 小 后 的 堆 地 址 小 于 堆 底 地 址， 则 不 进 行 缩 小
if new_pt <= heap_bottom {
return old_pt;
} else {
self.munmap(old_pt, increment as usize).unwrap();
}
}
new_pt
}
sbrk 的实现依赖于函数 mmap 的实现，将在接下来的章节介绍，mmap 是比 sbrk 更加灵
活的系统调用，能狗处理更复杂的内存分配和使用的情况。
pub fn sys_mmap(
start: usize,
len: usize,
prot: usize,
flags: usize,
fd: usize,
offset: usize,
) -> isize
参数 start：指向欲映射的内存起始地址，通常设为 NULL，代表让系统自动选定地址，映射成功后返回该地址。
• 参数 len：代表将文件中多大的部分映射到内存。
• 参数 prot：映射区域的保护方式。
• 参数 flags：影响映射区域的各种特性。在调用 mmap() 时必须要指定 MAP_-
SHARED 或 MAP_PRIVATE。MAP_FIXED 如果参数 start 所指的地址无法成功建
立映射时，则放弃映射，不对地址做修正。通常不鼓励用此旗标。MAP_SHARED
对映射区域的写入数据会复制回文件内，而且允许其他映射该文件的进程共享。
MAP_PRIVATE 对映射区域的写入操作会产生一个映射文件的复制，即私人的“写
入时复制”（copy on write）对此区域作的任何修改都不会写回原来的文件内容。
MAP_ANONYMOUS 建立匿名映射。此时会忽略参数 fd，不涉及文件，而且映射
区域无法和其他进程共享。MAP_DENYWRITE 只允许对映射区域的写入操作，其
他对文件直接写入的操作将会被拒绝。MAP_LOCKED 将映射区域锁定住，这表
示该区域不会被置换（swap）。
• 参数 fd：要映射到内存中的文件描述符。如果使用匿名内存映射时，即 flags 中设
置了 MAP_ANONYMOUS，fd 设为-1。
• 参数 offset：文件映射的偏移量，通常设置为 0，代表从文件最前方开始对应，offset
必须是分页大小的整数倍。
• 返回值：若映射成功则返回映射区的内存起始地址，否则返回-1。
mmap 用于把文件映射到用户空间中，简单说 mmap 就是把一个文件的内容在内存
里面做一个映像。映射成功后，用户对这段内存区域的修改可以直接反映到内核空间，
同样，内核空间对这段区域的修改也直接反映用户空间。那么对于内核空间 <—-> 用户
空间两者之间需要大量数据传输等操作的话效率是非常高的。进程可以像读写内存一样
对普通文件的操作。mmap 系统调用使得进程之间通过映射同一个普通文件实现共享内
存。UNIX 网络编程第二卷进程间通信对 mmap 函数进行了说明。该函数主要用途有三
个：1、将一个普通文件映射到内存中，通常在需要对文件进行频繁读写时使用，这样
用内存读写取代 I/O 读写，以获得较高的性能；2、将特殊文件进行匿名内存映射，可以
为关联进程提供共享内存空间；3、为无关联的进程提供共享内存空间，一般也是将一
个普通文件映射到内存中。第一步：找到最后一次 mmap 映射区域
```rust
let idx = self.last_mmap_area_idx();
每个进程所申请的用户地址空间是用 vector 存储的 vm_area_struct 的结构体，其中 start 和 end 就是代表这一个 area 虚拟地址的开始和结束，而我们的结构体在 vector 中是按照虚拟地址升序排序的，也就是说我们会找到最后一个符合要求的 vector 下标。

第二步：mmap 获取映射区域的起始地址，如果设置了 MAP_FIXED，则会直接使用用户的参数 start，不对地址做修正。
let start_va: VirtAddr = if flags.contains(MapFlags::MAP_FIXED) {
    // unmap if exists
    self.munmap(start, len);
    start.into()
} else {
    if let Some(idx) = idx {
        let area = &mut self.areas[idx];
        if flags.contains(MapFlags::MAP_PRIVATE | MapFlags::MAP_ANONYMOUS)
        && prot == area.map_perm
        && area.map_file.is_none() {

            let end_va: VirtAddr = area.inner.vpn_range.get_end().into();
            area.expand_to(VirtAddr::from(end_va.0 + len)).unwrap();
            return end_va.0 as isize;
        }
        area.inner.vpn_range.get_end().into()
    } else {
        MMAP_BASE.into()
    }
};
let mut new_area = MapArea::new(
    start_va,
    VirtAddr::from(start_va.0 + len),
    MapType::Framed,
    prot,
    None,
);
如果没有设置 MAP_FIXED，mmap 就会拿到先前得到的最后一次 mmap 映射区域。如果设置了 MAP_PRIVATE 或者 MAP_ANONYMOUS，就直接将要新映射的区域 new_area 与最后一次 mmap 映射区域合并并返回，没有设置就会将最后一次 mmap 映射区域的末尾作为新映射的区域的起始。而新映射的区域的末尾则根据用户参数 len 设置。

第三步：mmap 会判断是否是文件映射，即是否包含 MAP_ANONYMOUS。如果没有包含，就会先获取 fd_table，并根据用户参数 fd 从 fd_table 找到对应的文件描述符，这里文件相关内容不作赘述。然后设置文件偏移量 offset 并置入 new_area 中。
if !flags.contains(MapFlags::MAP_ANONYMOUS) {
    let fd_table = task.files.lock();
    match fd_table.get_ref(fd) {
        Ok(file_descriptor) => {
            if !file_descriptor.readable() {
                return EACCES;
            }
            let file = file_descriptor.file.deep_clone();
            file.lseek(offset as isize, SeekWhence::SEEK_SET).unwrap();
            new_area.map_file = Some(file);
        }
        Err(errno) => return errno,
    }
}
第四步：mmap 将 new_area 加入到先前的 vector 中，而且需要保持原 vector 的有序性，这里使用了 rust 语言的特性。
let (idx, _) = self
    .areas
    .iter()
    .enumerate()
    .skip_while(|&(_, area)| {
        area.inner.vpn_range.get_start() >= VirtAddr::from(MMAP_END).into()
    })
    .find(|&(_, area)| area.inner.vpn_range.get_start() >= start_va.into())
    .unwrap();
self.areas.insert(idx, new_area);
execve
应用程序自身的角度来看，进程 (Process) 的一个经典定义是一个正在运行的程序实例。当程序运行在操作系统中的时候，从程序的视角来看，它会产生一种“幻觉”：即该程序是整个计算机系统中当前运行的唯一的程序，能够独占使用处理器、内存和外设，而且程序中的代码和数据是系统内存中唯一的对象。体表现为“进程”这个抽象概念。站在计算机系统和操作系统的角度来看，并不存在这种“幻觉”。事实上，在一段时间之内，往往会有多个程序同时或交替在操作系统上运行，因此程序并不能独占整个计算机系统。具体而言，进程是应用程序的一次执行过程。并且在这个执行过程中，由“操作系统”执行环境来管理程序执行过程中的 进程上下文–一种控制流上下文。这里的进程上下文是指程序在运行中的各种物理/虚拟资源（寄存器、可访问的内存区域、打开的文件、信号等）的内容，特别是与程序执行相关的具体内容：内存中的代码和数据，栈、堆、当前执行的指令位置（程序计数器的内容）、当前执行时刻的各个通用寄存器中的值等。

exec 用一个新的程序来代替当前进程的内存和寄存器，但是其文件描述符、进程 id 和父进程都是不变的。它根据文件系统中保存的某个文件来初始化用户部分。‘exec‘通过 ‘open()‘打开二进制文件。然后，它读取 ELF 头。应用程序以通行的 ELF 格式来描述。一个 ELF 二进制文件包括了一个 ELF 头，然后是连续几个程序段的头。exec 会检查文件是否包含 ELF 二进制代码。一个 ELF 二进制文件是以 4 个“魔法数字”开头的，即 0x7F，“E”，“L”，“F”。如果 ELF 头中包含正确的魔法数字，‘exec‘ 就会认为该二进制文件的结构是正确的。
pub fn sys_execve(
    pathname: *const u8,
    mut argv: *const *const u8,
    mut envp: *const *const u8,
) -> isize
execve() 的前置条件 fork() 复制某一个进程，为 execve() 的执行，准备条件。execve() 的核心工作：参数转换，将 argv,envp 转为 Vec<String>。open() 打开 path 路径下的可执行文件，对其进行检查。load_elf() 将其加载到当前的空间中。load_elf 的核心工作：将 elf 文件映射到内核空间的 ‘MMAP_BASE‘地址处。在 map_elf() 中，把 elf 文件的内容拷贝到用户的 MemorySet 中。
let mut argv_vec: Vec<String> = Vec::with_capacity(16);
let mut envp_vec: Vec<String> = Vec::with_capacity(16);
if !argv.is_null() {
    loop {
        let arg_ptr = match translated_ref(token, argv) {
            Ok(argv) => *argv,
            Err(errno) => return errno,
        };
        if arg_ptr.is_null() {
            break;
        }
        argv_vec.push(match translated_str(token, arg_ptr) {
            Ok(arg) => arg,
            Err(errno) => return errno,
        });
        unsafe {
            argv = argv.add(1);
        }
    }
}
if !envp.is_null() {
    loop {
        let env_ptr = match translated_ref(token, envp) {
            Ok(envp) => *envp,
            Err(errno) => return errno,
        };
        if env_ptr.is_null() {
            break;
        }
        envp_vec.push(match translated_str(token, env_ptr) {
            Ok(env) => env,
            Err(errno) => return errno,
        });
        unsafe {
            envp = envp.add(1);
        }
    }
}
debug 层信息输出
debug!(
    "[exec] argv: {:?} /* {} vars */, envp: {:?} /* {} vars */",
    argv_vec,
    argv_vec.len(),
    envp_vec,
    envp_vec.len()
);
准备将 ELF 文件的内容载入当前进程要对读取的文件做以下检查 1、文件大小大于等于 4 2、同时，ELF 文件以 4 位魔数 \x7fELF 开头，因此应检查文件首是否有此魔数。
文件检查
match working_inode.open(&path, OpenFlags::O_RDONLY, false) {
    Ok(file) => {
        if file.get_size() < 4 {
            return ENOEXEC;
        }
        let mut magic_number = Box::<[u8; 4]>::new([0; 4]);
        // this operation may be expensive... I’m not sure
        file.read(Some(&mut 0usize), magic_number.as_mut_slice());
        let elf = match magic_number.as_slice() {
            b”\x7fELF” => file,
            b”#!” => {
                let shell_file = working_inode
                .open(DEFAULT_SHELL , OpenFlags::O_RDONLY, false
                ).unwrap();
                argv_vec.insert(0, DEFAULT_SHELL.to_string());
                shell_file
            }
            _ => return ENOEXEC,
        };
    }
    Err(errno) => errno,
}
真正的载入过程。load_elf 将当前的 elf 文件内容覆盖当前的进程。show_frame_consumption! 宏输出对应的信息
let task = current_task().unwrap();
show_frame_consumption! {
    "load_elf";
    if let Err(errno) = task.load_elf(elf, &argv_vec, &envp_vec) {
        return errno;
    };
}
// should return 0 in success
SUCCESS
clone() 从 linux 2.3.3 开始，glibc 的 ‘fork()‘封装作为 NPTL(Native POSIX Threads Library) 线程实现的一部分。直接调用 ‘fork()‘等效于调用 [clone(2)](https://man7.org/linux/man-pages/man2/clone.2.html) 时仅指定 ‘flags‘为 ‘SIGCHLD‘(共享信号句柄表)。创建线程的函数 ‘pthread_create‘内部使用的也是 clone 函数。在 glibc 的 ‘/sysdeps/unix/sysv/linux/createthread.c‘源码中可以看到，创建线程的函数 ‘create_thread‘中使用了 clone 函数，并指定了相关的 ‘flags‘：
const int clone_flags = (CLONE_VM | CLONE_FS
    | CLONE_FILES | CLONE_SYSVSEM
    | CLONE_SIGHAND | CLONE_THREAD
    | CLONE_SETTLS | CLONE_PARENT_SETTID
    | CLONE_CHILD_CLEARTID | 0);
    clone 的使用
    pub fn sys_clone(
    flags: u32,
    stack: *const u8,
    ptid: *mut u32,
    tls: usize,
    ctid: *mut u32,
) -> isize
描述该系统调用用于创建一个新的子进程，类似 fork(2)。与 fork(2) 相比，它可以更精确地控制调用进程和子进程之间的执行上下文细节。例如，使用这些系统调用，调用者可以控制两个进程之间是否共享虚拟地址空间，文件描述符表以及信号句柄表等。也可以通过这些系统调用将子进程放到不同的命名空间中。参数 flags: 包括 CloneFlags 和 exit_signal 两部分：子进程结束信号 exit_signal 当子进程退出时，会像父进程发送一个信号。退出信号在 clone() 的 flags 的低字节中指定如果该信号不是 SIGCHLD ，那么父进程在使用 wait(2) 等待子进程退出时必须指定 __WALL 或 WCLONE 选项。如果没有指定任何信号 (即，0)，则在子进程退出后不会向父进程发送任何信号。

以下是内核中如何从 flags 中获取子进程结束信号的代码块
获取子进程结束信号的代码块
let exit_signal = match Signals::from_signum((flags & 0xff) as usize) {
    Ok(signal) => signal,
    Err(_) => {
        warn!(
            "[sys_clone] signum of exit_signal is unspecified or
            invalid: {}",
            (flags & 0xff) as usize
        );
        // This is permitted by standard, but we only support 64
        signals
        Signals::empty()
    }
};
CloneFlags CloneFlags 用来指定 clone 系统调用的行为，内核的 CloneFlags 中有如下几种值
bitflags! {
    pub struct CloneFlags: u32 {
        //const CLONE_NEWTIME = 0x00000080;
        const CLONE_VM = 0x00000100;
        const CLONE_FS = 0x00000200;
        const CLONE_FILES = 0x00000400;
        const CLONE_SIGHAND = 0x00000800;
        const CLONE_PIDFD = 0x00001000;
        const CLONE_PTRACE = 0x00002000;
        const CLONE_VFORK = 0x00004000;
        const CLONE_PARENT = 0x00008000;
        const CLONE_THREAD = 0x00010000;
        const CLONE_NEWNS = 0x00020000;
        const CLONE_SYSVSEM = 0x00040000;
        const CLONE_SETTLS = 0x00080000;
        const CLONE_PARENT_SETTID = 0x00100000;
        const CLONE_CHILD_CLEARTID = 0x00200000;
        const CLONE_DETACHED = 0x00400000;
        const CLONE_UNTRACED = 0x00800000;
        const CLONE_CHILD_SETTID = 0x01000000;
        const CLONE_NEWCGROUP = 0x02000000;
        const CLONE_NEWUTS = 0x04000000;
        const CLONE_NEWIPC = 0x08000000;
        const CLONE_NEWUSER = 0x10000000;
        const CLONE_NEWPID = 0x20000000;
        const CLONE_NEWNET = 0x40000000;
        const CLONE_IO = 0x80000000;
    }
}
下文将介绍几个和传入参数有关的 cloneflags stack：stack 参数指定了子进程使用的栈的位置。由于子进程和调用进程可能会共享内存，因此不能在调用进程的栈中运行子进程。调用进程必须为子进程的栈配置内存空间，并向 clone() 传入一个执行该空间的指针。运行的所有处理器的栈都是向下生长的，因此 stack 通常指向为子进程栈设置的内存空间的最顶端地址。注意，clone() 没有为调用者提供一种可以将堆栈区域的大小通知内核的方法。

ptid，ctid：与 CloneFlags 中的 CLONE_CHILD_SETTID 和 CLONE_PARENT_SETTID 有关 CLONE_CHILD_SETTID 在 ctid 的位置上保存线程 ID。保存操作会在 clone 调用返回控制到子进程的用户空间前完成。（注意，在 clone 调用返回父进程前，保存操作可能是未完成的，它与是否引入 CLONE_VM 标志相关） CLONE_PARENT_SETTID 在父进程的 ptid 中保存子线程 ID。在 Linux 2.5.32-2.5.48 版本中，有一个标志 CLONE_SETTID 做了同样的事情。保存操作会在 clone 调用将控制返回给用户空间前完成。
tid：与 CloneFlags 中的 CLONE_SETTLS 有关 CLONE_SETTLS 将 TLS(Thread Local Storage) 保存到 tls 字段中。
clone 的具体实现
let child = parent.sys_clone(flags, stack, tls, exit_signal);
内核中 clone 的具体实现实际是实现在 TCB 结构体的方法 sys_clone 里，这里就是调用了 parent（也就是 clone 的调用者）的 sys_clone，它返回新创建的进程的 TCB 结构体首先，clone 会检查 CloneFlags 中是否有 CLONE_VM 和 CLONE_THREAD 如果有 CLONE_VM ，则调用进程和子进程会运行在系统的同一个内存空间中。调用进程或子进程对内存的写操作都可以被对方看到。如果没有设置 CLONE_VM ，则子进程会运行在执行 clone 时的调用进程的一份内存空间的拷贝中。如果有 CLONE_THREAD ，子线程会放到与调用进程相同的线程组中。当一个 clone 调用没有指定 CLONE_THREAD 时，生成的线程会放到一个新的线程组中，其 TGID 等于该线程的 TID。
CLONE_THREAD 的实现
let memory_set = if flags.contains(CloneFlags::CLONE_VM) {
    self.vm.clone()
} else {
    crate::mm::frame_reserve(16);
    Arc::new(Mutex::new(MemorySet::from_existing_user(
        &mut self.vm.lock(),
    )))
};
let tid_allocator = if flags.contains(CloneFlags::CLONE_THREAD) {
    self.tid_allocator.clone()
} else {
    Arc::new(Mutex::new(RecycleAllocator::new()))
};
if flags.contains(CloneFlags::CLONE_THREAD) {
    memory_set.lock().alloc_user_res(tid, stack.is_null());
}
接着 clone 会像操作系统申请 pid，tid，tgid 和内核栈，因为 npucore 模仿了 linux 的“以轻量级进程代替线程”，所以这里的 pid 与 tid 相同，为线程号，而 tgid 代表该进程的进程号。
线程号设置
let pid_handle = pid_alloc();
let tid = tid_allocator.lock().alloc();
let tgid = if flags.contains(CloneFlags::CLONE_THREAD) {
    self.tgid
} else {
    pid_handle.0
};
let kstack = kstack_alloc();
let kstack_top = kstack.get_top();
然后 clone 会进行新进程 TCB 的构造
let task_control_block = Arc::new(TaskControlBlock {
    pid: pid_handle,
    tid,
    tgid,
    kstack,
    ustack_base: if !stack.is_null() {
        stack as usize
    } else {
        ustack_bottom_from_tid(tid)
    },
    exit_signal ,
    exe: self.exe.clone(),
    tid_allocator ,
    files: if flags.contains(CloneFlags::CLONE_FILES) {
        self.files.clone()
    } else {
        Arc::new(Mutex::new(self.files.lock().clone()))
    },
    fs: if flags.contains(CloneFlags::CLONE_FS) {
        self.fs.clone()
    } else {
        Arc::new(Mutex::new(self.fs.lock().clone()))
    },
    vm: memory_set,
    sighand: if flags.contains(CloneFlags::CLONE_SIGHAND) {
        self.sighand.clone()
    } else {
        Arc::new(Mutex::new(self.sighand.lock().clone()))
    },
    futex: if flags.contains(CloneFlags::CLONE_SYSVSEM) {
        self.futex.clone()
    } else {
Arc::new(Mutex::new(Futex::new()))
},
inner: Mutex::new(TaskControlBlockInner {
// inherited
pgid: parent_inner.pgid,
heap_bottom: parent_inner.heap_bottom ,
heap_pt: parent_inner.heap_pt,
// clone
sigpending: parent_inner.sigpending.clone(),
// new
children: Vec::new(),
rusage: Rusage::new(),
clock: ProcClock::new(),
clear_child_tid: 0,
robust_list: RobustList::default(),
timer: [ITimerVal::new(); 3],
sigmask: Signals::empty(),
// compute
trap_cx_ppn ,
task_cx: TaskContext::goto_trap_return(kstack_top),
parent: if flags.contains(CloneFlags::CLONE_PARENT)
| flags.contains(CloneFlags::CLONE_THREAD) {
parent_inner.parent.clone()
} else {
Some(Arc::downgrade(self))
},
// constants
task_status: TaskStatus::Ready,
exit_code: 0,
}),
});
} else {
Arc::new(Mutex::new(Futex::new()))
},
inner: Mutex::new(TaskControlBlockInner {
// inherited
pgid: parent_inner.pgid,
heap_bottom: parent_inner.heap_bottom ,
heap_pt: parent_inner.heap_pt,
// clone
sigpending: parent_inner.sigpending.clone(),
// new
children: Vec::new(),
rusage: Rusage::new(),
clock: ProcClock::new(),
clear_child_tid: 0,
robust_list: RobustList::default(),
timer: [ITimerVal::new(); 3],
sigmask: Signals::empty(),
// compute
trap_cx_ppn ,
task_cx: TaskContext::goto_trap_return(kstack_top),
parent: if flags.contains(CloneFlags::CLONE_PARENT)
| flags.contains(CloneFlags::CLONE_THREAD) {
parent_inner.parent.clone()
} else {
Some(Arc::downgrade(self))
},
// constants
task_status: TaskStatus::Ready,
exit_code: 0,
}),
});
这里对相关 CloneFlags 进行说明 CLONE_FILES 子进程与父进程共享相同的文件描述符（file descriptor）表 CLONE_FS 子进程与父进程共享相同的文件系统，包括 root、当前目录、umask CLONE_SIGHAND 子进程与父进程共享相同的信号处理（signal handler）表 CLONE_SYSVSEM 如果设置了该标志，则子进程和调用进程会共享一组 System V semaphore adjustment (semadj) 值 （参见 semop(2))。这种情况下，共享列表会在共享该列表的所有进程之间累加 semadj 值，并且仅当共享列表的最后一个进程终止（或使用 unshare(2) 停止共享列表）时才会执行 semaphore adjustments。如果没有设置该标志，则子进程会有一个独立的 semadj 列表，且初始为空。与信号量操作有关。

信号量操作
```rust
files: if flags.contains(CloneFlags::CLONE_FILES) {
    self.files.clone()
} else {
    Arc::new(Mutex::new(self.files.lock().clone()))
},
fs: if flags.contains(CloneFlags::CLONE_FS) {
    self.fs.clone()
} else {
    Arc::new(Mutex::new(self.fs.lock().clone()))
},
sighand: if flags.contains(CloneFlags::CLONE_SIGHAND) {
    self.sighand.clone()
} else {
    Arc::new(Mutex::new(self.sighand.lock().clone()))
},
futex: if flags.contains(CloneFlags::CLONE_SYSVSEM) {
    self.futex.clone()
} else {
    Arc::new(Mutex::new(Futex::new()))
},
这里会修改父进程的子进程表，如果设置了 CLONE_PARENT 或 CLONE_THREAD 则代表新建进程与调用进程是兄弟关系，反之则调用进程是新建进程的父进程
// add child
if flags.contains(CloneFlags::CLONE_PARENT) || flags.contains(
CloneFlags::CLONE_THREAD) {
    if let Some(grandparent) = &parent_inner.parent {
        grandparent
        .upgrade()
        .unwrap()
        .acquire_inner_lock()
        .children
        .push(task_control_block.clone());
    }
} else {
    parent_inner.children.push(task_control_block.clone());
}
最后，clone 会设置新进程的上下文以及 tls 并返回

设置新进程的上下文
let trap_cx = task_control_block.acquire_inner_lock().get_trap_cx();
if flags.contains(CloneFlags::CLONE_THREAD) {
    *trap_cx = *parent_inner.get_trap_cx();
}
// we also do not need to prepare parameters on stack, musl has done it
for us
if !stack.is_null() {
    trap_cx.gp.sp = stack as usize;
}
// set tp
if flags.contains(CloneFlags::CLONE_SETTLS) {
    trap_cx.gp.tp = tls;
}
// for child process, fork returns 0
trap_cx.gp.a0 = 0;
// modify kernel_sp in trap_cx
trap_cx.kernel_sp = kstack_top;
// return
task_control_block
进程切换

进程切换是操作系统中的一项核心功能，其作用是在多任务操作系统中实现进程间的切换，使得每个进程都可以独立地运行并且感觉到自己在独占 CPU。

实现进程的切换具有以下几个重要作用：

提高系统的并发性：进程切换可以使得多个进程在同一时刻并发运行，从而提高系统的效率和资源利用率。

实现多任务操作系统的核心功能：进程切换是多任务操作系统的核心功能之一，它可以使得每个进程都独立运行，并且感觉到自己在独占 CPU。

总之，进程切换是操作系统中一项重要的功能，它可以提高系统的并发性和效率，支持多任务和多用户系统，并且是操作系统实现任务调度、资源分配等核心功能的基础。

(1) 进程切换的基本流程

进程的切换大致可以分为以下几个步骤：

用户态陷入内核态（系统调用或中断）
切换到调度器进程
切换到新进程的内核线程
从内核态返回
为了在进程之间进行切换，内核需要在内核态执行两次上下文切换：从旧进程的内核线程切换到 CPU 的调度器线程，以及从调度器线程切换到新进程的内核线程。进程切换的核心 __switch 并不了解线程，它只是简单地保存和恢复寄存器集合，即上下文。上下文是 CPU 保存的当前进程执行的状态。对于 RISC-V，进程上下文包括：ra 进程的返回地址、sp 进程内核栈指针、s0-s11 被调用者保存寄存器。

Register ABI Name Description Saver
x0 zero Hard-wired zero —
x1 ra Return address Caller
x2 sp Stack pointer Callee
x3 gp Global pointer —
x4 tp Thread pointer —
x5-7 t0-2 Temporaries Caller
x8 s0/fp Saved register/frame pointer Callee
x9 s1 Saved register Callee
x10-11 a0-1 Function arguments/return values Caller
x12-17 a2-7 Function arguments Caller
x18-27 s2-11 Saved registers Callee
x28-31 t3-6 Temporaries Caller
f0-7 ft0-7 FP temporaries Caller
f8-9 fs0-1 FP saved registers Callee
f10-11 fa0-1 FP arguments/return values Caller
f12-17 fa2-7 FP arguments Caller
f18-27 fs2-11 FP saved registers Callee
f28-31 ft8-11 FP temporaries Caller

(2) 实现进程切换的方法

进程的切换依靠 os/src/task/mod.rs 中的 suspend_current_and_run_next 函数实现。该函数的应用主要有如下两个场景：

• 应用程序手动使用 sys_yield 系统调用让出当前进程的占用权。
pub fn sys_yield() -> isize {
    suspend_current_and_run_next();
    SUCCESS
}
 内核触发定时中断
 Trap::Interrupt(Interrupt::SupervisorTimer) => {
    do_wake_expired();
    set_next_trigger();
    suspend_current_and_run_next();
}
uspend_current_and_run_next 函数实现如下：
pub fn suspend_current_and_run_next() {
    // There must be an application running.
    let task = take_current_task().unwrap();

    // ‐‐‐‐ hold current PCB lock
    let mut task_inner = task.acquire_inner_lock();
    let task_cx_ptr = &mut task_inner.task_cx as *mut TaskContext;
    // Change status to Ready
    task_inner.task_status = TaskStatus::Ready;
    drop(task_inner);
    // ‐‐‐‐ release current PCB lock

    // push back to ready queue.
    add_task(task);
    // jump to scheduling cycle
    schedule(task_cx_ptr);
}
函数首先获取了一个当前正在执行进程的 PCB ，命名为 task 。然后从刚才得到的任务 PCB 里面的可变部分（类型为使用 MutexGuard 锁保护住的一个泛型结构体 MutexGuard<TaskControlBlockInner> ），获取当前进程的 task_cx （类型是一个结构体 TaskContext ，保存着当前进程的上下文信息)，然后把他转化成一个可变的裸指针。再当前进程的状态从“正在运行”改为“就绪”，也就是停止当前进程。接着手动调用 drop 让 task_inner 的引用计数值减一，并调用了 add_task 方法将进程加入就绪队列。最后调用 schedule 函数完成进程的切换。

TaskContext 的定义如下。ra 为调用后需要返回位置的 pc 值，它记录了 __switch 函数返回之后应该跳转到哪里继续执行，从而在任务切换完成并 ret 之后能到正确的位置；sp 为当前程序用户栈的栈指针；s0 s11 是需要保存的 12 个寄存器。
pub struct TaskContext {
    ra: usize,
    sp: usize
    ,
    s: [usize; 12],
}

add_task 方法是将当前的线程加入到懒分配的全局变量 TASK_MANAGER 的就绪队列中。

代码片段 5.22 add task
```rust
lazy_static! {
    pub static ref TASK_MANAGER: Mutex<TaskManager> = Mutex::new(
        TaskManager::new());
}

pub fn add_task(task: Arc<TaskControlBlock>) {
    TASK_MANAGER.lock().add(task);
}

impl TaskManager {
    pub fn add(&mut self, task: Arc<TaskControlBlock>) {
        self.ready_queue.push_back(task);
    }
}schedule 函数负责将当前的进程切换到处理器调度进程 (idle task)。处理器调度进程由懒分配的全局变量 PROCESSOR 管理，切换过程通过汇编代码 __switch 实现。

TaskContext
pub struct Processor {
    current: Option<Arc<TaskControlBlock>>,
    idle_task_cx: TaskContext,
}

lazy_static! {
    pub static ref PROCESSOR: Mutex<Processor> = Mutex::new(Processor::new());
}

pub fn schedule(switched_task_cx_ptr: *mut TaskContext) {
    let idle_task_cx_ptr = PROCESSOR.lock().get_idle_task_cx_ptr();
    unsafe {
        __switch(switched_task_cx_ptr, idle_task_cx_ptr);
    }
}
__switch 函数接受两个参数，旧进程的内核栈指针和新进程的内核栈指针。它将旧进程的上下文 (sp,ra,s0-s11) 保存到内存中，然后从内存从取出新进程的上下文到 CPU 的寄存器中。

__switch
.altmacro
.macro SAVE_SN n
sd s\n, (\n+2)*8(a0)
.endm
.macro LOAD_SN n
ld s\n, (\n+2)*8(a1)
.endm
.section .text
.globl __switch
__switch:
# __switch(
# current_task_cx_ptr: *mut TaskContext,
# next_task_cx_ptr: *const TaskContext
# )
# save kernel stack of current task
sd sp, 8(a0)
# save ra & s0~s11 of current execution
sd ra, 0(a0)
.set n, 0
.rept 12
SAVE_SN %n
.set n, n + 1
.endr
# restore ra & s0~s11 of next execution
ld ra, 0(a1)
.set n, 0
.rept 12
LOAD_SN %n
.set n, n + 1
.endr
# restore kernel stack of next task
ld sp, 8(a1)
ret
文件系统
块设备接口层为文件系统提供了对块设备进行读写的操作接口，其代码在 easy-fs/src/block_dev.rs 中。作为 easy-fs 库的最底层，其声明了一个块设备的抽象接口 BlockDevice，该 trait 定义了设备驱动所需要实现的读写接口，如下：

easy-fs/src/block_dev.rs
pub trait BlockDevice: Send + Sync + Any {
    /// Read block from BlockDevice
    /// The function panics when the size of ’buf’ is not a multiple of
    BLOCK_SZ
    fn read_block(&self, block_id: usize, buf: &mut [u8]);

    /// Write block into the file system.
    /// The function panics when the size of ’buf’ is not a multiple of
    BLOCK_SZ
    fn write_block(&self, block_id: usize, buf: &[u8]);

    /// We should rewrite the API for K210 since it supports NATIVE multi‐
    block clearing
    fn clear_block(&self, block_id: usize, num: u8) {
        self.write_block(block_id, &[num; BLOCK_SZ]);
    }

    /// We should rewrite the API for K210 if it supports NATIVE multi‐
    block clearing
    fn clear_mult_block(&self, block_id: usize, cnt: usize, num: u8) {
        for i in block_id..block_id + cnt {
            self.write_block(i, &[num; BLOCK_SZ]);
        }
    }
}
中对于文件系统同样实现了 buff cache 层，其主要定义于 easy-fs/src/block_cache.rs 文件中。

首先是对于整个块缓存的定义：

Cache 缓存对象基本操作
pub trait Cache {

    fn read<T, V>(&self, offset: usize, f: impl FnOnce(&T) -> V) -> V;

    fn modify<T, V>(&mut self, offset: usize, f: impl FnOnce(&mut T) -> V) -> V;

    fn sync(&self, _block_ids: Vec<usize>, _block_device: &Arc<dyn BlockDevice>) {}
}
pub trait Cache {

    fn read<T, V>(&self, offset: usize, f: impl FnOnce(&T) -> V) -> V;

    fn modify<T, V>(&mut self, offset: usize, f: impl FnOnce(&mut T) -> V) -> V;

    fn sync(&self, _block_ids: Vec<usize>, _block_device: &Arc<dyn BlockDevice>) {}
}
我们实现了一个 CacheManager 对于缓存进行更好的管理，第一点是可以统一管理不同类型的缓存，如块缓存和页表缓存，其次可以实现资源管理和性能优化，内核中对 oom 的具体实现的是综合了引用计数和优先级两种策略的方法，这有助于提高系统的整体性能，减少磁盘 I/O 操作。

CacheManager 的实现
pub trait CacheManager {
    /// The constant to mark the cache size.
    const CACHE_SZ: usize;

    type CacheType: Cache;

    /// Constructor to the struct.
    fn new() -> Self
    where
        Self: Sized;

    /// Tell cache manager to write back cache and release memory
    /// Argument：
    /// ’neighbor’: A closure to get block ids when cache miss.
    /// ’block_device’: The pointer to the block_device.
    /// Return Value：
    /// Number of caches freed
    fn oom<FUNC>(
        &self,
        _neighbor: FUNC,
        _block_device: &Arc<dyn BlockDevice>
    ) -> usize
    where
        FUNC: Fn(usize) -> Vec<usize>
    {
        unreachable!()
    }

    /// When file size changed, we should notify cache manager to drop some
    cache
    /// Argument：
    /// ’new_size’: File’s new size
    fn notify_new_size(
        &self,
        _new_size: usize
    ) {
        unreachable!()
    }
}
这个 trait 定义了缓存管理器的基本功能。

new() -> Self: 静态方法，用于创建一个新的缓存管理器实例。

oom<FUNC>(&self, _neighbor: FUNC, _block_device: &Arc<dyn BlockDevice>) -> usize: 当系统内存不足时，释放一些缓存。接受一个邻居闭包来获取块 ID，以及一个指向块设备的引用。

notify_new_size(&self, _new_size: usize): 当文件大小变化时，通知缓存管理器释放一些缓存块。

然后就是对于缓存管理的分别具体实现，因为我们有块缓存和页缓存两种缓存块，所以我们这里分别对 CacheManager 进一步封装实现了 BlockCacheManager 和 PageManager：

BCM 和 CM
pub trait BlockCacheManager: CacheManager {
    /// Try to get the block cache and return None if not found.
    /// Argument：
    /// block_id’: The demanded block id(for block cache).
    fn try_get_block_cache(
        &self,
        block_id: usize,
    ) -> Option<Arc<Mutex<Self::CacheType >>>;

    /// Attempt to get block cache from the cache.
    /// If failed, the manager should try to copy the block from sdcard.
    /// Argument：
    /// block_id’: The demanded block id(for block cache).
    /// neighbor’: A closure to get block ids when cache miss.
    /// block_device’: The pointer to the block_device.
    /// Return Value：
    /// Pointer to cache
    fn get_block_cache<FUNC>(
        &self,
        block_id: usize,
        block_device: &Arc<dyn BlockDevice>,
    ) -> Arc<Mutex<Self::CacheType>>;
}
遇见问题
万事开头难，在实验的提交阶段我遇见了很多问题。

首先就是第一次提交，提示文件内容过大，删除掉 riskv_64 文件夹后可以正常提交。

第二就是第二次提交，显示了 Compile Error，经过对问题的分析，和与朋友的交流，问题的来源是，系统没有实现 shut_down 调用。实现了之后就进入到下一步。

第三就是第三次提交，显示 Compile Error，分析得出 makefile 文件中 ifelse 缺少，通过借鉴其他同学的 makefile 文件，解决了问题。

第四就是 Makefile:1: *** missing separator. Stop. 通过使用 gpt 搜索，得知是 makefile 文件在第一行缺少了分隔符。在 Makefile 中，每个规则都由一个目标（target），一个或多个依赖（prerequisites），以及一个或多个命令（commands）组成。这些部分之间需要用制表符（Tab）作为分隔符。

如何运行
进入文件目录后依次输入
make all

qemu-system-riscv64 -machine virt -kernel kernel-qemu -m 128M -nographic -smp 2 -bios sbi-qemu -drive file=sdcard.img,if=none,format=raw,id=x0  -device virtio-blk-device,drive=x0,bus=virtio-mmio-

### 运行结果

```plaintext
========== START test_openat ==========
open dir fd: 3
openat fd: 4
openat success.
========== END test_openat ==========
========== START test_dup2 ==========
 from fd 100
========== END test_dup2 ==========
========== START test_pipe ==========
cpid: 2
cpid: 0
 Write to pipe successfully.

========== END test_pipe ==========
========== START test_getppid ==========
 getppid success. ppid : 1
========== END test_getppid ==========
========== START test_brk ==========
Before alloc,heap pos: 8192
After alloc,heap pos: 8256
Alloc again,heap pos: 8320
========== END test_brk ==========
========== START test_chdir ==========
chdir ret: 0
 current working dir : /test_chdir
========== END test_chdir ==========
========== START test_clone ==========
 Child says successfully!
clone process successfully.
pid:2
========== END test_clone ==========
========== START test_close ==========
 close 3 success.
========== END test_close ==========
========== START test_dup ==========
 new fd is 3.
========== END test_dup ==========
========== START test_execve ==========
 I am test_echo.
execve success.
========== END test_execve ==========
========== START test_exit ==========
exit OK.
========== END test_exit ==========
========== START test_fork ==========
 child process.
 parent process. wstatus:0
========== END test_fork ==========
========== START test_fstat ==========
fstat ret: 0
fstat: dev: 2048, inode: 7368, mode: 33279, nlink: 1, size: 52, atime: 0, mtime: 0, ctime: 0
========== END test_fstat ==========
========== START test_getcwd ==========
getcwd: / successfully!
========== END test_getcwd ==========
========== START test_getdents ==========
open fd:3
getdents fd:456
getdents success.
brk
========== END test_getdents ==========
========== START test_getpid ==========
getpid success.
pid = 1
========== END test_getpid ==========
========== START test_gettimeofday ==========
gettimeofday success.
start:569, end:588
interval: 19
========== END test_gettimeofday ==========
========== START test_mkdir ==========
mkdir ret: 0
 mkdir success.
========== END test_mkdir ==========
========== START test_mmap ==========
file len: 27
mmap content:   Hello, mmap successfully!
========== END test_mmap ==========
========== START test_mount ==========
Mounting dev:/dev/vda2 to ./mnt
mount return: 0
mount successfully
umount return: 0
========== END test_mount ==========
========== START test_munmap ==========
file len: 27
munmap return: 0
munmap successfully!
========== END test_munmap ==========
========== START test_open ==========
Hi, this is a text file.
syscalls testing success!
========== END test_open ==========
========== START test_read ==========
Hi, this is a text file.
syscalls testing success!
========== END test_read ==========
========== START test_times ==========
mytimes success
{tms_utime:37987, tms_stime:212300, tms_cutime:0, tms_cstime:0}
========== END test_times ==========
========== START test_umount ==========
Mounting dev:/dev/vda2 to ./mnt
mount return: 0
umount success.
return: 0
========== END test_umount ==========
========== START test_uname ==========
Uname: Linux debian 5.10.0-7-riscv64 #1 SMP Debian 5.10.40-1 (2021-05-28) riscv64 
========== END test_uname ==========
========== START test_unlink ==========
 unlink success!
========== END test_unlink ==========
========== START test_wait ==========
This is child process
wait child success.
wstatus: 0
========== END test_wait ==========
========== START test_waitpid ==========
This is child process
waitpid successfully.
wstatus: 3
========== END test_waitpid ==========
========== START test_write ==========
Hello operating system contest.
========== END test_write ==========
========== START test_yield ==========
 I am child process: 2. iteration 0.
 I am child process: 3. iteration 1.
 I am child process: 4. iteration 2.
 I am child process: 2. iteration 0.
 I am child process: 3. iteration 1.
 I am child process: 4. iteration 2.
 I am child process: 2. iteration 0.
 I am child process: 3. iteration 1.
 I am child process: 4. iteration 2.
 I am child process: 2. iteration 0.
 I am child process: 3. iteration 1.
 I am child process: 4. iteration 2.
========== END test_yield ==========
========== START test_sleep ==========
sleep success.
========== END test_sleep ==========


